"""PvE Views"""

# Django
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.humanize.templatetags.humanize import intcomma
from django.core.exceptions import ValidationError
from django.core.handlers.wsgi import WSGIRequest
from django.db import IntegrityError, transaction
from django.http import JsonResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required
from allianceauth.authentication.models import UserProfile
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from esi.decorators import token_required

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA TaxSystem
from taxsystem import __title__, forms, tasks
from taxsystem.api.helpers import (
    get_character_permissions,
    get_corporation,
    get_manage_corporation,
    get_manage_permission,
)
from taxsystem.helpers.views import add_info_to_context
from taxsystem.models.filters import JournalFilter, JournalFilterSet
from taxsystem.models.logs import AdminLogs, PaymentHistory
from taxsystem.models.tax import Members, OwnerAudit, Payments, PaymentSystem

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required
@permission_required("taxsystem.basic_access")
def index(request: WSGIRequest):
    """Index View"""
    return redirect(
        "taxsystem:payments", request.user.profile.main_character.corporation_id
    )


@login_required
@permission_required("taxsystem.basic_access")
def admin(request: WSGIRequest):
    corporation_id = request.user.profile.main_character.corporation_id
    if not request.user.is_superuser:
        messages.error(request, _("You do not have permission to access this page."))
        return redirect("taxsystem:index")

    if request.method == "POST":
        force_refresh = False
        if request.POST.get("force_refresh", False):
            force_refresh = True
        if request.POST.get("run_clear_etag"):
            messages.info(request, _("Queued Clear All ETags"))
            tasks.clear_all_etags.apply_async(priority=1)
        if request.POST.get("run_taxsystem_updates"):
            messages.info(request, _("Queued Update All Taxsystem"))
            tasks.update_all_taxsytem.apply_async(
                kwargs={"force_refresh": force_refresh}, priority=7
            )
    context = {
        "corporation_id": corporation_id,
        "title": _("Tax System Superuser Administration"),
    }
    return render(request, "taxsystem/admin.html", context=context)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
def administration(request: WSGIRequest, corporation_id: int):
    """Manage View"""
    context = {
        "corporation_id": corporation_id,
        "title": _("Administration"),
        "forms": {
            "accept_request": forms.PaymentAcceptForm(),
            "reject_request": forms.PaymentRejectForm(),
            "add_request": forms.PaymentAddForm(),
            "payment_delete_request": forms.PaymentDeleteForm(),
            "undo_request": forms.PaymentUndoForm(),
            "switchuser_request": forms.TaxSwitchUserForm(),
            "delete_request": forms.MemberDeleteForm(),
        },
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/manage.html", context=context)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
def manage_filter(request: WSGIRequest, corporation_id: int):
    """Manage View"""
    owner, perms = get_manage_corporation(request, corporation_id)

    filter_sets = owner.ts_filter_set.all()
    context = {
        "corporation_id": corporation_id,
        "filter_sets": filter_sets,
        "title": _("Manage Filters"),
        "forms": {
            "filter": forms.AddJournalFilterForm(queryset=owner.ts_filter_set.all()),
            "filter_set": forms.CreateFilterSetForm(),
            "delete_request": forms.MemberDeleteForm(),
        },
    }
    if perms is False:
        messages.error(
            request, _("You do not have permission to manage this corporation.")
        )
        return redirect("taxsystem:index")

    with transaction.atomic():
        form_add = forms.AddJournalFilterForm(
            data=request.POST, queryset=owner.ts_filter_set.all()
        )
        form_set = forms.CreateFilterSetForm(data=request.POST)

        if form_add.is_valid():
            queryset = form_add.cleaned_data["filter_set"]
            filter_type = form_add.cleaned_data["filter_type"]
            value = form_add.cleaned_data["value"]
            try:
                JournalFilter.objects.create(
                    filter_set=queryset,
                    filter_type=filter_type,
                    value=value,
                )
            except IntegrityError:
                messages.error(request, _("A filter with this name already exists."))
                return redirect(
                    "taxsystem:manage_filter", corporation_id=corporation_id
                )
            except Exception as e:  # pylint: disable=broad-except
                messages.error(
                    request, _("Something went wrong, please try again later.")
                )
                logger.error("Error creating journal filter: %s", e)
                return redirect(
                    "taxsystem:manage_filter", corporation_id=corporation_id
                )

        if form_set.is_valid():
            name = form_set.cleaned_data["name"]
            description = form_set.cleaned_data["description"]
            try:
                JournalFilterSet.objects.create(
                    owner=owner,
                    name=name,
                    description=description,
                )
            except IntegrityError:
                messages.error(
                    request, _("A filter set with this name already exists.")
                )
                return redirect(
                    "taxsystem:manage_filter", corporation_id=corporation_id
                )
            except Exception as e:  # pylint: disable=broad-except
                messages.error(
                    request, _("Something went wrong, please try again later.")
                )
                logger.error("Error creating journal filter set: %s", e)
                return render(request, "taxsystem/manage-filter.html", context=context)

    return render(request, "taxsystem/manage-filter.html", context=context)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
def switch_filterset(request: WSGIRequest, corporation_id: int, filter_set_id: int):
    """Deactivate Filter Set View"""
    owner, perms = get_manage_corporation(request, corporation_id)

    if perms is False:
        messages.error(
            request, _("You do not have permission to manage this corporation.")
        )
        return redirect("taxsystem:index")

    filter_set = get_object_or_404(owner.ts_filter_set, id=filter_set_id)
    filter_sets = owner.ts_filter_set.all()

    filter_set.enabled = not filter_set.enabled
    filter_set.save()

    context = {
        "corporation_id": corporation_id,
        "filter_sets": filter_sets,
        "forms": {
            "filter": forms.AddJournalFilterForm(queryset=owner.ts_filter_set.all()),
            "filter_set": forms.CreateFilterSetForm(),
        },
        "title": _("Deactivate Filter Set"),
    }
    context = add_info_to_context(request, context)

    messages.success(
        request, _(f"Filter set switched to {filter_set.enabled} successfully.")
    )
    return redirect("taxsystem:manage_filter", corporation_id=corporation_id)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
def delete_filterset(request: WSGIRequest, corporation_id: int, filter_set_id: int):
    """Delete Filter Set View"""
    owner, perms = get_manage_corporation(request, corporation_id)

    if perms is False:
        messages.error(
            request, _("You do not have permission to manage this corporation.")
        )
        return redirect("taxsystem:index")

    filter_set = get_object_or_404(owner.ts_filter_set, id=filter_set_id)
    filter_sets = owner.ts_filter_set.all()

    filter_set.delete()
    msg = _(f"{filter_set.name} from {owner.name} deleted")
    AdminLogs(
        user=request.user,
        owner=owner,
        action=AdminLogs.Actions.DELETE,
        comment=msg,
    ).save()
    messages.success(request, _("Filter set deleted successfully."))

    context = {
        "corporation_id": corporation_id,
        "filter_sets": filter_sets,
        "forms": {
            "filter": forms.AddJournalFilterForm(queryset=owner.ts_filter_set.all()),
            "filter_set": forms.CreateFilterSetForm(),
        },
        "title": _("Delete Filter Set"),
    }
    context = add_info_to_context(request, context)

    return redirect("taxsystem:manage_filter", corporation_id=corporation_id)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@require_POST
def delete_filter(request: WSGIRequest, corporation_id: int, filter_pk: int):
    msg = _("Invalid Method")
    corp = get_corporation(request, corporation_id)

    perms = get_manage_permission(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    form = forms.FilterDeleteForm(data=request.POST)
    if form.is_valid():
        filter_obj = JournalFilter.objects.get(filter_set__owner=corp, pk=filter_pk)
        if filter_obj:
            msg = _(
                f"{filter_obj.filter_type}({filter_obj.value}) from {filter_obj.filter_set} deleted - {form.cleaned_data['delete_reason']}"
            )
            filter_obj.delete()
            AdminLogs(
                user=request.user,
                owner=corp,
                action=AdminLogs.Actions.DELETE,
                comment=msg,
            ).save()
            return JsonResponse(
                data={"success": True, "message": msg}, status=200, safe=False
            )
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
def edit_filterset(request: WSGIRequest, corporation_id: int, filter_set_id: int):
    """Edit Filter Set View"""
    owner, perms = get_manage_corporation(request, corporation_id)

    if perms is False:
        messages.error(
            request, _("You do not have permission to manage this corporation.")
        )
        return redirect("taxsystem:index")

    edit_set = get_object_or_404(owner.ts_filter_set, id=filter_set_id)
    filter_sets = owner.ts_filter_set.all()

    if request.method == "POST":
        form = forms.EditFilterSetForm(request.POST, instance=edit_set)
        if form.is_valid():
            form.save()
            messages.success(request, _("Filter set updated successfully."))
            return redirect("taxsystem:manage_filter", corporation_id=corporation_id)
    else:
        form = forms.EditFilterSetForm(instance=edit_set)

    context = {
        "corporation_id": corporation_id,
        "filter_sets": filter_sets,
        "forms": {
            "edit_filter_set": form,
        },
        "title": _("Edit Filter Set"),
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/manage-filter.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def payments(request: WSGIRequest, corporation_id: int):
    """Payments View"""
    perms = get_corporation(request, corporation_id)

    if perms is None:
        messages.error(request, _("No Corporation found."))

    corporations = OwnerAudit.objects.visible_to(request.user)

    context = {
        "corporation_id": corporation_id,
        "title": _("Payments"),
        "forms": {
            "add_request": forms.PaymentAddForm(),
            "payment_delete_request": forms.PaymentDeleteForm(),
            "accept_request": forms.PaymentAcceptForm(),
            "reject_request": forms.PaymentRejectForm(),
            "undo_request": forms.PaymentUndoForm(),
        },
        "corporations": corporations,
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/payments.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def own_payments(request: WSGIRequest, corporation_id=None):
    """Own Payments View"""
    if corporation_id is None:
        corporation_id = request.user.profile.main_character.corporation_id

    corporations, perms = get_manage_corporation(request, corporation_id)

    if corporations is None:
        messages.error(request, _("No Corporation found."))

    if perms is False:
        messages.error(request, _("Permission Denied"))
        return redirect("taxsystem:index")

    corporations = OwnerAudit.objects.visible_to(request.user)

    context = {
        "corporation_id": corporation_id,
        "title": _("Own Payments"),
        "corporations": corporations,
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/own-payments.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def faq(request: WSGIRequest, corporation_id: int):
    """Payments View"""
    corporations = OwnerAudit.objects.visible_to(request.user)

    context = {
        "corporation_id": corporation_id,
        "title": _("FAQ"),
        "corporations": corporations,
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/faq.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def account(request: WSGIRequest, character_id=None):
    """Account View"""
    if character_id is None:
        character_id = request.user.profile.main_character.character_id
    logger.error(f"Character ID not provided, using main character ID: {character_id}")

    user_profile = UserProfile.objects.filter(
        main_character__character_id=character_id
    ).first()

    if not user_profile:
        messages.error(request, _("No User found."))
        return redirect("taxsystem:index")

    try:
        corporation_id = user_profile.main_character.corporation_id
        owner, perms = get_manage_corporation(request, corporation_id)
        perms = perms or get_character_permissions(request, character_id)
    except AttributeError:
        messages.error(request, _("User has no main character set."))
        return redirect("taxsystem:index")

    payment_user = PaymentSystem.objects.filter(
        user__profile=user_profile,
        owner=owner,
    ).first()

    if not payment_user:
        messages.error(request, _("No Payment System User found."))
        return redirect("taxsystem:index")

    if owner is None:
        messages.error(request, _("Corporation not Found"))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied"))
        return redirect("taxsystem:index")

    try:
        member = owner.ts_members.get(character_id=character_id)
    except owner.ts_members.model.DoesNotExist:
        member = None

    context = {
        "title": _("Account"),
        "character_id": character_id,
        "corporation_id": corporation_id,
        "account": {
            "name": payment_user.name,
            "corporation": owner,
            "status": payment_user.Status(payment_user.status).html(text=True),
            "deposit": (
                payment_user.deposit_html
                if payment_user.status != PaymentSystem.Status.MISSING
                else "N/A"
            ),
            "has_paid": (
                payment_user.has_paid_icon(badge=True, text=True)
                if payment_user.status != PaymentSystem.Status.MISSING
                else "N/A"
            ),
            "last_paid": (
                payment_user.last_paid
                if payment_user.status != PaymentSystem.Status.MISSING
                else "N/A"
            ),
            "joined": member.joined if member else "N/A",
            "last_login": member.logon if member else "N/A",
        },
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/account.html", context=context)


@login_required
@permission_required("taxsystem.create_access")
@token_required(scopes=OwnerAudit.get_esi_scopes())
def add_corp(request, token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)
    corp, __ = EveCorporationInfo.objects.get_or_create(
        corporation_id=char.corporation_id,
        defaults={
            "member_count": 0,
            "corporation_ticker": char.corporation_ticker,
            "corporation_name": char.corporation_name,
        },
    )

    owner, created = OwnerAudit.objects.update_or_create(
        corporation=corp,
        defaults={
            "name": corp.corporation_name,
            "active": True,
        },
    )

    if created:
        AdminLogs(
            user=request.user,
            owner=owner,
            action=AdminLogs.Actions.ADD,
            comment=_("Added to Tax System"),
        ).save()

    tasks.update_corporation.apply_async(
        args=[owner.pk], kwargs={"force_refresh": True}, priority=6
    )
    msg = _("{corporation_name} successfully added/updated to Tax System").format(
        corporation_name=corp.corporation_name,
    )
    messages.info(request, msg)
    return redirect("taxsystem:index")


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@require_POST
def approve_payment(request: WSGIRequest, corporation_id: int, payment_pk: int):
    msg = _("Invalid Method")
    corp = get_corporation(request, corporation_id)

    perms = get_manage_permission(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentAcceptForm(data=request.POST)
            if form.is_valid():
                reason = form.cleaned_data["accept_info"]
                payment = Payments.objects.get(account__owner=corp, pk=payment_pk)
                if payment.is_pending or payment.is_needs_approval:
                    msg = _(
                        "Payment ID: {pid} - Amount: {amount} - Name: {name} approved"
                    ).format(
                        pid=payment.pk,
                        amount=intcomma(payment.amount),
                        name=payment.name,
                    )
                    payment.request_status = Payments.RequestStatus.APPROVED
                    payment.reviser = request.user.profile.main_character.character_name
                    payment.save()

                    payment_account = PaymentSystem.objects.get(
                        owner=corp, user=payment.account.user
                    )
                    payment_account.deposit += payment.amount
                    payment_account.save()
                    PaymentHistory(
                        user=request.user,
                        payment=payment,
                        action=PaymentHistory.Actions.STATUS_CHANGE,
                        comment=reason,
                        new_status=Payments.RequestStatus.APPROVED,
                    ).save()
                    return JsonResponse(
                        data={"success": True, "message": msg}, status=200, safe=False
                    )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@require_POST
def undo_payment(request: WSGIRequest, corporation_id: int, payment_pk: int):
    msg = _("Invalid Method")
    corp = get_corporation(request, corporation_id)

    perms = get_manage_permission(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentUndoForm(data=request.POST)
            if form.is_valid():
                reason = form.cleaned_data["undo_reason"]
                payment = Payments.objects.get(account__owner=corp, pk=payment_pk)
                if payment.is_approved or payment.is_rejected:
                    msg = _(
                        "Payment ID: {pid} - Amount: {amount} - Name: {name} undone"
                    ).format(
                        pid=payment.pk,
                        amount=intcomma(payment.amount),
                        name=payment.name,
                    )
                    # Ensure that the payment is not rejected
                    if not payment.is_rejected:
                        payment_account = PaymentSystem.objects.get(
                            owner=corp, user=payment.account.user
                        )
                        payment_account.deposit -= payment.amount
                        payment_account.save()
                    payment.request_status = Payments.RequestStatus.PENDING
                    payment.reviser = ""
                    payment.save()
                    PaymentHistory(
                        user=request.user,
                        payment=payment,
                        action=PaymentHistory.Actions.STATUS_CHANGE,
                        comment=reason,
                        new_status=Payments.RequestStatus.PENDING,
                    ).save()
                    return JsonResponse(
                        data={"success": True, "message": msg}, status=200, safe=False
                    )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@require_POST
def reject_payment(request: WSGIRequest, corporation_id: int, payment_pk: int):
    msg = _("Invalid Method")
    corp = get_corporation(request, corporation_id)

    perms = get_manage_permission(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentRejectForm(data=request.POST)
            if form.is_valid():
                reason = form.cleaned_data["reject_reason"]
                payment = Payments.objects.get(account__owner=corp, pk=payment_pk)
                if payment.is_pending or payment.is_needs_approval:
                    payment.request_status = Payments.RequestStatus.REJECTED
                    payment.reviser = request.user.profile.main_character.character_name
                    payment.save()

                    payment_account = PaymentSystem.objects.get(
                        owner=corp, user=payment.account.user
                    )
                    payment_account.save()
                    msg = _(
                        "Payment ID: {pid} - Amount: {amount} - Name: {name} rejected"
                    ).format(
                        pid=payment.pk,
                        amount=intcomma(payment.amount),
                        name=payment.name,
                    )

                    PaymentHistory(
                        user=request.user,
                        payment=payment,
                        action=PaymentHistory.Actions.STATUS_CHANGE,
                        comment=reason,
                        new_status=Payments.RequestStatus.REJECTED,
                    ).save()
                    return JsonResponse(
                        data={"success": True, "message": msg}, status=200, safe=False
                    )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@require_POST
def delete_payment(request: WSGIRequest, corporation_id: int, payment_pk: int):
    msg = _("Invalid Method")
    corp = get_corporation(request, corporation_id)

    perms = get_manage_permission(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentDeleteForm(data=request.POST)
            if form.is_valid():
                reason = form.cleaned_data["delete_reason"]
                payment = Payments.objects.get(account__owner=corp, pk=payment_pk)

                if payment.entry_id != 0:  # Prevent deletion of ESI imported payments
                    msg = _(
                        "Payment ID: {pid} - Amount: {amount} - Name: {name} deletion failed - ESI imported payments cannot be deleted"
                    ).format(
                        pid=payment.pk,
                        amount=intcomma(payment.amount),
                        name=payment.name,
                    )
                    return JsonResponse(
                        data={"success": False, "message": msg}, status=400, safe=False
                    )

                msg = _(
                    "Payment ID: {pid} - Amount: {amount} - Name: {name} deleted - {reason}"
                ).format(
                    pid=payment.pk,
                    amount=intcomma(payment.amount),
                    name=payment.name,
                    reason=reason,
                )

                # Refund if approved
                if payment.is_approved:
                    payment_user = PaymentSystem.objects.get(
                        owner=corp, user=payment.account.user
                    )
                    payment_user.deposit -= payment.amount
                    payment_user.save()
                # Delete Payment
                payment.delete()

                # Log Payment Action
                AdminLogs(
                    user=request.user,
                    owner=corp,
                    action=AdminLogs.Actions.DELETE,
                    comment=msg,
                ).save()

                return JsonResponse(
                    data={"success": True, "message": msg}, status=200, safe=False
                )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@require_POST
def add_payment(request: WSGIRequest, corporation_id: int, payment_system_pk: int):
    msg = _("Invalid Method")
    corp = get_corporation(request, corporation_id)

    perms = get_manage_permission(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentAddForm(data=request.POST)
            if form.is_valid():
                amount = form.cleaned_data["amount"]
                reason = form.cleaned_data["add_reason"]
                payment_account = PaymentSystem.objects.get(
                    owner=corp, pk=payment_system_pk
                )

                payment = Payments(
                    name=payment_account.user.username,
                    entry_id=0,  # Manual Entry
                    amount=amount,
                    account=payment_account,
                    date=timezone.now(),
                    reason=reason,
                    request_status=Payments.RequestStatus.APPROVED,
                    reviser=request.user.username,
                    corporation_id=corporation_id,
                )
                payment.save()
                payment_account.deposit += amount
                payment_account.save()

                msg = _(
                    "Payment ID: {pid} - Amount: {amount} - Name: {name} added"
                ).format(
                    pid=payment.pk,
                    amount=intcomma(payment.amount),
                    name=payment.name,
                )

                PaymentHistory(
                    user=request.user,
                    payment=payment,
                    action=PaymentHistory.Actions.PAYMENT_ADDED,
                    comment=reason,
                    new_status=Payments.RequestStatus.APPROVED,
                ).save()

                # Log Admin Action
                AdminLogs(
                    user=request.user,
                    owner=corp,
                    action=AdminLogs.Actions.ADD,
                    comment=msg,
                ).save()

                return JsonResponse(
                    data={"success": True, "message": msg}, status=200, safe=False
                )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@require_POST
def switch_user(request: WSGIRequest, corporation_id: int, payment_system_pk: int):
    msg = _("Invalid Method")
    corp = get_corporation(request, corporation_id)

    perms = get_manage_permission(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.TaxSwitchUserForm(data=request.POST)
            if form.is_valid():
                payment_system = PaymentSystem.objects.get(
                    owner=corp, pk=payment_system_pk
                )
                if payment_system.is_active:
                    payment_system.status = PaymentSystem.Status.DEACTIVATED
                    msg = _("Payment System User: %s deactivated") % payment_system.name
                else:
                    payment_system.status = PaymentSystem.Status.ACTIVE
                    msg = _("Payment System User: %s activated") % payment_system.name

                AdminLogs(
                    user=request.user,
                    owner=corp,
                    action=AdminLogs.Actions.CHANGE,
                    comment=msg,
                ).save()
                payment_system.save()
            return JsonResponse(
                data={"success": True, "message": msg}, status=200, safe=False
            )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@csrf_exempt
def update_tax_amount(request: WSGIRequest, corporation_id: int):
    if request.method == "POST":
        value = float(request.POST.get("value"))
        msg = _("Please enter a valid number")
        try:
            if value < 0:
                return JsonResponse({"message": msg}, status=400)
        except ValueError:
            return JsonResponse({"message": msg}, status=400)

        corp = get_corporation(request, corporation_id)

        perms = get_manage_permission(request, corporation_id)

        if not perms:
            return JsonResponse({"message": _("Permission Denied")}, status=403)

        try:
            corp.tax_amount = value
            corp.save()
            msg = _(f"Tax Amount from {corp.name} updated to {value}")
            AdminLogs(
                user=request.user,
                owner=corp,
                action=AdminLogs.Actions.CHANGE,
                comment=msg,
            ).save()
        except ValidationError:
            return JsonResponse({"message": msg}, status=400)
        return JsonResponse({"message": msg}, status=200)
    return JsonResponse({"message": _("Invalid request method")}, status=405)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@csrf_exempt
def update_tax_period(request: WSGIRequest, corporation_id: int):
    if request.method == "POST":
        value = int(request.POST.get("value"))
        msg = _("Please enter a valid number")
        try:
            if value < 0:
                return JsonResponse({"message": msg}, status=400)
        except ValueError:
            return JsonResponse({"message": msg}, status=400)

        corp = get_corporation(request, corporation_id)

        perms = get_manage_permission(request, corporation_id)

        if not perms:
            return JsonResponse({"message": _("Permission Denied")}, status=403)

        try:
            corp.tax_period = value
            corp.save()
            msg = _(f"Tax Period from {corp.name} updated to {value}")
            AdminLogs(
                user=request.user,
                owner=corp,
                action=AdminLogs.Actions.CHANGE,
                comment=msg,
            ).save()
        except ValidationError:
            return JsonResponse({"message": msg}, status=400)
        return JsonResponse({"message": msg}, status=200)
    return JsonResponse({"message": _("Invalid request method")}, status=405)


@login_required
@permissions_required(["taxsystem.manage_own_corp", "taxsystem.manage_corps"])
@require_POST
def delete_member(request: WSGIRequest, corporation_id: int, member_pk: int):
    msg = _("Invalid Method")
    corp = get_corporation(request, corporation_id)

    perms = get_manage_permission(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    form = forms.MemberDeleteForm(data=request.POST)
    if form.is_valid():
        reason = form.cleaned_data["delete_reason"]
        member = Members.objects.get(owner=corp, pk=member_pk)
        if member.is_missing:
            msg = _(f"Member {member.character_name} deleted - {reason}")
            member.delete()
            AdminLogs(
                user=request.user,
                owner=corp,
                action=AdminLogs.Actions.DELETE,
                comment=msg,
            ).save()
            return JsonResponse(
                data={"success": True, "message": msg}, status=200, safe=False
            )
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)



# Django
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext as _

# AA TaxSystem
from taxsystem.api.helpers import generate_button
from taxsystem.models.tax import Members


def _delete_member(corporation_id, member: Members, perms, request):
    # Check if user has permission to view the actions
    if not perms:
        return ""

    template = "taxsystem/partials/form/button.html"
    url = reverse(
        viewname="taxsystem:delete_member",
        kwargs={"corporation_id": corporation_id, "member_pk": member.pk},
    )

    if member.is_missing:
        confirm_text = (
            _("Are you sure to Confirm")
            + f"?<br><span class='fw-bold'>{member.character_name} "
            + _("Delete")
            + "</span>"
        )

        settings_data = {
            "title": _("Delete Member"),
            "icon": "fas fa-trash",
            "color": "danger",
            "text": confirm_text,
            "modal": "members-delete-member",
            "action": url,
            "ajax": "action",
        }
        # Generate the buttons
        actions = []
        actions.append(
            generate_button(corporation_id, template, member, settings_data, request)
        )

        actions_html = format_html("".join(actions))
        return format_html(
            '<div class="d-flex justify-content-end">{}</div>', actions_html
        )
    return ""



# Django
from django.contrib.auth.models import Permission
from django.test import TestCase

# Alliance Auth
from allianceauth.tests.auth_utils import AuthUtils

# Alliance Auth (External Libs)
from app_utils.testing import create_user_from_evecharacter

# AA TaxSystem
from taxsystem.models.tax import OwnerAudit
from taxsystem.tests.testdata.generate_owneraudit import (
    create_owneraudit_from_user,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth

MODULE_PATH = "taxsystem.models.owneraudit"


class TestOwnerAuditModel(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()

        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001, permissions=["taxsystem.basic_access"]
        )
        cls.user2, cls.character_ownership2 = create_user_from_evecharacter(
            1002, permissions=["taxsystem.basic_access"]
        )
        cls.audit = create_owneraudit_from_user(cls.user)
        cls.audit2 = create_owneraudit_from_user(cls.user2)

    def test_str(self):
        expected_str = OwnerAudit.objects.get(id=self.audit.pk)
        self.assertEqual(self.audit, expected_str)

    def test_get_esi_scopes(self):
        self.assertEqual(
            self.audit.get_esi_scopes(),
            [
                # General
                "esi-corporations.read_corporation_membership.v1",
                "esi-corporations.track_members.v1",
                "esi-characters.read_corporation_roles.v1",
                # wallets
                "esi-wallet.read_corporation_wallets.v1",
                "esi-corporations.read_divisions.v1",
            ],
        )

    def test_access_no_perms(self):
        """Test should return only own corporation if basic_access is set."""
        corporation = OwnerAudit.objects.visible_to(self.user)
        self.assertIn(self.audit, corporation)
        self.assertNotIn(self.audit2, corporation)

    def test_access_perms_own_corp(self):
        """Test should return only own corporation if manage_own_corp is set."""
        self.user = AuthUtils.add_permission_to_user_by_name(
            "taxsystem.manage_own_corp", self.user
        )
        self.user.refresh_from_db()
        corporation = OwnerAudit.objects.visible_to(self.user)
        self.assertIn(self.audit, corporation)
        self.assertNotIn(self.audit2, corporation)

    def test_access_perms_manage_corps(self):
        """Test should return all corporations if manage_corps is set."""
        self.user = AuthUtils.add_permission_to_user_by_name(
            "taxsystem.manage_corps", self.user
        )
        self.user.refresh_from_db()
        corporation = OwnerAudit.objects.visible_to(self.user)
        self.assertIn(self.audit, corporation)
        self.assertIn(self.audit2, corporation)

    def test_manage_to_no_perms(self):
        """Test should return None if no permissions are set."""
        corporation = OwnerAudit.objects.manage_to(self.user)
        self.assertNotIn(self.audit, corporation)
        self.assertNotIn(self.audit2, corporation)

    def test_manage_to_perms_own_corp(self):
        """Test should return only own corporation if manage_own_corp is set."""
        self.user = AuthUtils.add_permission_to_user_by_name(
            "taxsystem.manage_own_corp", self.user
        )
        self.user.refresh_from_db()
        corporation = OwnerAudit.objects.manage_to(self.user)
        self.assertIn(self.audit, corporation)
        self.assertNotIn(self.audit2, corporation)

    def test_manage_to_perms_manage_corps(self):
        """Test should return all corporations if manage_corps is set."""
        self.user = AuthUtils.add_permission_to_user_by_name(
            "taxsystem.manage_corps", self.user
        )
        self.user.refresh_from_db()
        corporation = OwnerAudit.objects.manage_to(self.user)
        self.assertIn(self.audit, corporation)
        self.assertIn(self.audit2, corporation)



# Generated by Django 4.2.20 on 2025-05-19 06:52

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0004_owneraudit_last_update_payday_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="corporationwalletdivision",
            old_name="division",
            new_name="division_id",
        ),
        migrations.RemoveField(
            model_name="owneraudit",
            name="last_update_members",
        ),
        migrations.RemoveField(
            model_name="owneraudit",
            name="last_update_payday",
        ),
        migrations.RemoveField(
            model_name="owneraudit",
            name="last_update_payment_system",
        ),
        migrations.RemoveField(
            model_name="owneraudit",
            name="last_update_payments",
        ),
        migrations.RemoveField(
            model_name="owneraudit",
            name="last_update_wallet",
        ),
        migrations.AlterField(
            model_name="corporationwalletdivision",
            name="corporation",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_division",
                to="taxsystem.owneraudit",
            ),
        ),
        migrations.AlterField(
            model_name="members",
            name="corporation",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_members",
                to="taxsystem.owneraudit",
            ),
        ),
        migrations.AlterField(
            model_name="owneraudit",
            name="active",
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name="payments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_payments",
                to="taxsystem.paymentsystem",
            ),
        ),
        migrations.AlterField(
            model_name="paymentsystem",
            name="corporation",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_payment_system",
                to="taxsystem.owneraudit",
            ),
        ),
        migrations.RenameField(
            model_name="members",
            old_name="corporation",
            new_name="owner",
        ),
        migrations.RenameField(
            model_name="paymentsystem",
            old_name="corporation",
            new_name="owner",
        ),
        migrations.RenameField(
            model_name="adminlogs",
            old_name="corporation",
            new_name="owner",
        ),
        migrations.RenameField(
            model_name="smartgroup",
            old_name="corporation",
            new_name="owner",
        ),
        migrations.AlterField(
            model_name="smartgroup",
            name="owner",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_filter_sets",
                to="taxsystem.owneraudit",
            ),
        ),
        migrations.AlterField(
            model_name="corporationwalletjournalentry",
            name="division",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_wallet",
                to="taxsystem.corporationwalletdivision",
            ),
        ),
        migrations.CreateModel(
            name="OwnerUpdateStatus",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "section",
                    models.CharField(
                        choices=[
                            ("wallet", "Wallet Journal"),
                            ("division", "Wallet Division"),
                            ("members", "Members"),
                            ("payments", "Payments"),
                            ("payment_system", "Payment System"),
                            ("payday", "Payday"),
                        ],
                        db_index=True,
                        max_length=32,
                    ),
                ),
                (
                    "is_success",
                    models.BooleanField(db_index=True, default=None, null=True),
                ),
                ("error_message", models.TextField()),
                ("has_token_error", models.BooleanField(default=False)),
                (
                    "last_run_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_run_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_update_status",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
    ]



"""App URLs"""

# Django
from django.urls import path, re_path

# AA TaxSystem
from taxsystem import views
from taxsystem.api import api

app_name: str = "taxsystem"

urlpatterns = [
    # -- Tax System
    path("", views.index, name="index"),
    path("admin/", views.admin, name="admin"),
    path(
        "corporation/<int:corporation_id>/view/payments/",
        views.payments,
        name="payments",
    ),
    path(
        "corporation/<int:corporation_id>/view/own_payments/",
        views.own_payments,
        name="own_payments",
    ),
    path(
        "corporation/<int:corporation_id>/view/administration/",
        views.administration,
        name="administration",
    ),
    path(
        "corporation/<int:corporation_id>/view/filters/",
        views.manage_filter,
        name="manage_filter",
    ),
    path("corporation/<int:corporation_id>/view/faq/", views.faq, name="faq"),
    # --- Tax Administration
    # -- Tax Payments
    path("corporation/add/", views.add_corp, name="add_corp"),
    path(
        "corporation/<int:corporation_id>/payment/<int:payment_system_pk>/add/",
        views.add_payment,
        name="add_payment",
    ),
    path(
        "corporation/<int:corporation_id>/payment/<int:payment_pk>/delete/",
        views.delete_payment,
        name="delete_payment",
    ),
    path(
        "corporation/<int:corporation_id>/payment/<int:payment_pk>/approve/",
        views.approve_payment,
        name="approve_payment",
    ),
    path(
        "corporation/<int:corporation_id>/payment/<int:payment_pk>/undo/",
        views.undo_payment,
        name="undo_payment",
    ),
    path(
        "corporation/<int:corporation_id>/payment/<int:payment_pk>/reject/",
        views.reject_payment,
        name="reject_payment",
    ),
    # -- Tax Manage
    path(
        "corporation/<int:corporation_id>/manage/update_tax/",
        views.update_tax_amount,
        name="update_tax_amount",
    ),
    path(
        "corporation/<int:corporation_id>/manage/update_period/",
        views.update_tax_period,
        name="update_tax_period",
    ),
    path(
        "corporation/<int:corporation_id>/manage/member/<int:member_pk>/delete/",
        views.delete_member,
        name="delete_member",
    ),
    # -- Tax Manage Filters
    path(
        "corporation/<int:corporation_id>/manage/filter_set/<int:filter_set_id>/deactivate/",
        views.switch_filterset,
        name="switch_filterset",
    ),
    path(
        "corporation/<int:corporation_id>/manage/filter_set/<int:filter_set_id>/edit/",
        views.edit_filterset,
        name="edit_filterset",
    ),
    path(
        "corporation/<int:corporation_id>/manage/filter_set/<int:filter_set_id>/delete/",
        views.delete_filterset,
        name="delete_filterset",
    ),
    path(
        "corporation/<int:corporation_id>/manage/filter/<int:filter_pk>/delete/",
        views.delete_filter,
        name="delete_filter",
    ),
    # -- Tax Payment System
    path("corporation/view/account/", views.account, name="account"),
    path("corporation/view/account/<int:character_id>/", views.account, name="account"),
    path(
        "corporation/<int:corporation_id>/manage/user/<int:payment_system_pk>/switch_user/",
        views.switch_user,
        name="switch_user",
    ),
    # -- API System
    re_path(r"^api/", api.urls),
]



# Django
from django.contrib.humanize.templatetags.humanize import intcomma
from django.db.models import Count, F, Q
from django.utils import timezone
from django.utils.translation import gettext as _

# AA TaxSystem
from taxsystem.models.tax import Members, OwnerAudit, Payments, PaymentSystem
from taxsystem.models.wallet import CorporationWalletDivision


def _get_divisions_dict(divisions: CorporationWalletDivision):
    divisions_dict = {}
    total_balance = 0
    for i, division in enumerate(divisions, start=1):
        division_name = division.name if division.name else f"{i}. {_('Division')}"
        division_balance = intcomma(division.balance)
        divisions_dict[division_name] = {
            "name": division_name,
            "balance": division_balance,
        }
        total_balance += division.balance

    divisions_dict["total"] = {
        "name": _("Total"),
        "balance": intcomma(total_balance),
    }

    return divisions_dict


def _get_statistics_dict(owner: OwnerAudit):
    payments_counts = Payments.objects.filter(account__owner=owner).aggregate(
        total=Count("id"),
        automatic=Count("id", filter=Q(reviser="System")),
        manual=Count("id", filter=~Q(reviser="System") & ~Q(reviser="")),
        pending=Count(
            "id",
            filter=Q(
                request_status__in=[
                    Payments.RequestStatus.PENDING,
                    Payments.RequestStatus.NEEDS_APPROVAL,
                ]
            ),
        ),
    )

    period = timezone.timedelta(days=owner.tax_period)

    payment_system_counts = (
        PaymentSystem.objects.filter(
            owner=owner,
            user__profile__main_character__isnull=False,
        )
        .exclude(status=PaymentSystem.Status.MISSING)
        .aggregate(
            users=Count("id"),
            active=Count("id", filter=Q(status=PaymentSystem.Status.ACTIVE)),
            inactive=Count("id", filter=Q(status=PaymentSystem.Status.INACTIVE)),
            deactivated=Count("id", filter=Q(status=PaymentSystem.Status.DEACTIVATED)),
            paid=Count(
                "id",
                filter=Q(deposit__gte=F("owner__tax_amount"))
                & Q(status=PaymentSystem.Status.ACTIVE)
                | Q(deposit=0)
                & Q(status=PaymentSystem.Status.ACTIVE)
                & Q(last_paid__gte=timezone.now() - period),
            ),
        )
    )

    unpaid = payment_system_counts["active"] - payment_system_counts["paid"]

    members_count = Members.objects.filter(owner=owner).aggregate(
        total=Count("character_id"),
        unregistered=Count("character_id", filter=Q(status=Members.States.NOACCOUNT)),
        alts=Count("character_id", filter=Q(status=Members.States.IS_ALT)),
        mains=Count("character_id", filter=Q(status=Members.States.ACTIVE)),
    )

    return {
        # Payment System
        "payment_users": payment_system_counts["users"],
        "payment_users_active": payment_system_counts["active"],
        "payment_users_inactive": payment_system_counts["inactive"],
        "payment_users_deactivated": payment_system_counts["deactivated"],
        "payment_users_paid": payment_system_counts["paid"],
        "payment_users_unpaid": unpaid,
        # Payments
        "payments": payments_counts["total"],
        "payments_pending": payments_counts["pending"],
        "payments_auto": payments_counts["automatic"],
        "payments_manually": payments_counts["manual"],
        # Members
        "members": members_count["total"],
        "members_unregistered": members_count["unregistered"],
        "members_alts": members_count["alts"],
        "members_mains": members_count["mains"],
    }



# Standard Library
from unittest.mock import MagicMock, patch
from urllib import request

# Django
from django.test import override_settings
from django.utils import timezone

# Alliance Auth (External Libs)
from app_utils.testing import NoSocketsTestCase, create_user_from_evecharacter

# AA TaxSystem
from taxsystem.models.filters import JournalFilter
from taxsystem.models.tax import Payments, PaymentSystem
from taxsystem.tests.testdata.generate_filter import create_filter, create_filterset
from taxsystem.tests.testdata.generate_owneraudit import create_owneraudit_from_user
from taxsystem.tests.testdata.generate_payments import (
    create_member,
    create_payment,
    create_payment_system,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "taxsystem.managers.wallet_manager"


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestPaymentsManager(NoSocketsTestCase):
    """Test Payments Manager for Corporation Journal Entries."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
        )

        cls.audit = create_owneraudit_from_user(
            user=cls.user,
            tax_amount=1000,
            tax_period=30,
        )

        cls.payment_system = create_payment_system(
            name=cls.character_ownership.character.character_name,
            owner=cls.audit,
            user=cls.user,
            status=PaymentSystem.Status.ACTIVE,
            deposit=0,
            last_paid=(timezone.now() - timezone.timedelta(days=30)),
        )

        cls.payments = create_payment(
            name=cls.character_ownership.character.character_name,
            account=cls.payment_system,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
            reason="Tax Payment",
            request_status=Payments.RequestStatus.PENDING,
            reviser="",
        )

        cls.payments2 = create_payment(
            name=cls.character_ownership.character.character_name,
            account=cls.payment_system,
            entry_id=2,
            amount=6000,
            date=timezone.datetime(2025, 1, 1, 14, 0, 0, tzinfo=timezone.utc),
            reason="Mining Stuff",
            request_status=Payments.RequestStatus.PENDING,
            reviser="",
        )

        cls.filter_set = create_filterset(
            owner=cls.audit,
            name="100m",
            description="Filter for payments over 100m",
        )

        cls.filter_amount = create_filter(
            filter_set=cls.filter_set,
            filter_type=JournalFilter.FilterType.AMOUNT,
            value=1000,
        )

    def test_update_payments(self):
        # given

        self.audit.update_payment_system(force_refresh=False)

        self.assertSetEqual(
            set(self.payment_system.ts_payments.values_list("entry_id", flat=True)),
            {1, 2},
        )
        obj = self.payment_system.ts_payments.get(entry_id=1)
        self.assertEqual(obj.amount, 1000)
        self.assertEqual(obj.request_status, Payments.RequestStatus.APPROVED)

        obj = self.payment_system.ts_payments.get(entry_id=2)
        self.assertEqual(obj.amount, 6000)
        self.assertEqual(obj.request_status, Payments.RequestStatus.NEEDS_APPROVAL)



"""TestView class."""

# Standard Library
import json
from http import HTTPStatus

# Django
from django.test import RequestFactory, TestCase
from django.urls import reverse

# Alliance Auth (External Libs)
from app_utils.testing import create_user_from_evecharacter

# AA TaxSystem
from taxsystem import views
from taxsystem.models.tax import Members
from taxsystem.tests.testdata.generate_owneraudit import (
    create_owneraudit_from_user,
)
from taxsystem.tests.testdata.generate_payments import create_member
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "taxsystem.views"


class TestDeleteMember(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            character_id=1001,
            permissions=[
                "taxsystem.basic_access",
                "taxsystem.manage_own_corp",
            ],
        )
        cls.audit = create_owneraudit_from_user(cls.user)
        cls.no_audit_user, _ = create_user_from_evecharacter(
            character_id=1002,
            permissions=[
                "taxsystem.basic_access",
                "taxsystem.manage_own_corp",
            ],
        )
        cls.no_permission_user, _ = create_user_from_evecharacter(
            character_id=1003,
            permissions=[
                "taxsystem.basic_access",
            ],
        )
        cls.member = create_member(
            character_name="Gneuten",
            character_id=1001,
            owner=cls.audit,
            status=Members.States.MISSING,
        )

    def test_delete_member(self):
        """Test delete member."""
        corporation_id = self.audit.corporation.corporation_id
        member_pk = self.member.pk

        form_data = {
            "corporation_id": corporation_id,
            "confirm": "yes",
            "delete_reason": "Test reason",
        }

        request = self.factory.post(
            reverse("taxsystem:delete_member", args=[corporation_id, member_pk]),
            data=form_data,
        )

        request.user = self.user

        response = views.delete_member(
            request, corporation_id=corporation_id, member_pk=member_pk
        )

        response_data = json.loads(response.content)

        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertTrue(response_data["success"])
        self.assertEqual(
            response_data["message"],
            f"Member {self.member.character_name} deleted - Test reason",
        )

    def test_no_permission(self):
        """Test try undo a payment without permission."""
        corporation_id = self.audit.corporation.corporation_id
        member_pk = self.member.pk

        form_data = {
            "corporation_id": corporation_id,
            "confirm": "yes",
            "delete_reason": "Test reason",
        }

        request = self.factory.post(
            reverse("taxsystem:delete_member", args=[corporation_id, member_pk]),
            data=form_data,
        )

        request.user = self.no_audit_user

        response = views.delete_member(
            request, corporation_id=corporation_id, member_pk=member_pk
        )

        response_data = json.loads(response.content)

        self.assertEqual(response.status_code, HTTPStatus.FORBIDDEN)
        self.assertFalse(response_data["success"])
        self.assertEqual(response_data["message"], "Permission Denied")

    def test_no_manage_permission(self):
        """Test undo payment without managing permission."""
        corporation_id = self.audit.corporation.corporation_id
        member_pk = self.member.pk

        form_data = {
            "corporation_id": corporation_id,
            "confirm": "yes",
            "delete_reason": "Test reason",
        }

        request = self.factory.post(
            reverse("taxsystem:delete_member", args=[corporation_id, member_pk]),
            data=form_data,
        )

        request.user = self.no_permission_user

        response = views.delete_member(
            request, corporation_id=corporation_id, member_pk=member_pk
        )

        self.assertEqual(response.status_code, HTTPStatus.FOUND)



# AA Tax System
# AA TaxSystem
from taxsystem.models.tax import Members, OwnerAudit, Payments, PaymentSystem


def create_payment(account: PaymentSystem, **kwargs) -> Payments:
    """Create a Payment for a Corporation"""
    params = {
        "account": account,
    }
    params.update(kwargs)
    payment = Payments(**params)
    payment.save()
    return payment


def create_member(owner: OwnerAudit, **kwargs) -> Members:
    """Create a Payment System for a Corporation"""
    params = {
        "owner": owner,
    }
    params.update(kwargs)
    member = Members(**params)
    member.save()
    return member


def create_payment_system(owner: OwnerAudit, **kwargs) -> PaymentSystem:
    """Create a Payment System for a Corporation"""
    params = {
        "owner": owner,
    }
    params.update(kwargs)
    payment_system = PaymentSystem(**params)
    payment_system.save()
    return payment_system



"""TestView class."""

# Standard Library
from http import HTTPStatus
from math import exp
from unittest.mock import Mock, patch

# Django
from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase
from django.urls import reverse

# Alliance Auth (External Libs)
from app_utils.testing import create_user_from_evecharacter

# AA TaxSystem
from taxsystem import views
from taxsystem.models.filters import JournalFilter
from taxsystem.tests.testdata.generate_filter import create_filter, create_filterset
from taxsystem.tests.testdata.generate_owneraudit import (
    create_owneraudit_from_user,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "taxsystem.views"


class TestSwitchSetFilter(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            character_id=1001,
            permissions=[
                "taxsystem.basic_access",
                "taxsystem.manage_own_corp",
            ],
        )
        cls.audit = create_owneraudit_from_user(cls.user)
        cls.no_audit_user, _ = create_user_from_evecharacter(
            character_id=1002,
            permissions=[
                "taxsystem.basic_access",
                "taxsystem.manage_own_corp",
            ],
        )
        cls.no_permission_user, _ = create_user_from_evecharacter(
            character_id=1003,
            permissions=[
                "taxsystem.basic_access",
            ],
        )

        cls.filter_set = create_filterset(
            owner=cls.audit,
            name="100m",
            description="Filter for payments over 100m",
        )

        cls.filter_amount = create_filter(
            filter_set=cls.filter_set,
            filter_type=JournalFilter.FilterType.AMOUNT,
            value=1000,
        )

    def _message_middleware(self, request):
        """Middleware to add a message to the response."""
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)

    @patch(MODULE_PATH + ".messages")
    def test_switch_filterset(self, mock_messages):
        """Test switch filter."""
        corporation_id = self.audit.corporation.corporation_id
        filterset_id = self.filter_set.id

        request = self.factory.get(
            reverse("taxsystem:switch_filterset", args=[corporation_id, filterset_id]),
        )
        request.user = self.user

        self._message_middleware(request)

        response = views.switch_filterset(
            request, corporation_id=corporation_id, filter_set_id=filterset_id
        )

        expected_status = not self.filter_set.enabled

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.success.assert_called_once_with(
            request, f"Filter set switched to {expected_status} successfully."
        )

    @patch(MODULE_PATH + ".messages")
    def test_no_permission(self, mock_messages):
        """Test try switch without permission."""
        corporation_id = self.audit.corporation.corporation_id
        filterset_id = self.filter_set.id

        request = self.factory.get(
            reverse("taxsystem:switch_filterset", args=[corporation_id, filterset_id]),
        )

        request.user = self.no_audit_user

        self._message_middleware(request)

        response = views.switch_filterset(
            request, corporation_id=corporation_id, filter_set_id=filterset_id
        )

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called_once_with(
            request, "You do not have permission to manage this corporation."
        )

    def test_no_manage_permission(self):
        """Test switch without managing permission."""
        corporation_id = self.audit.corporation.corporation_id
        filterset_id = self.filter_set.id

        request = self.factory.get(
            reverse("taxsystem:switch_filterset", args=[corporation_id, filterset_id]),
        )

        request.user = self.no_permission_user

        response = views.switch_filterset(
            request, corporation_id=corporation_id, filter_set_id=filterset_id
        )

        self.assertEqual(response.status_code, HTTPStatus.FOUND)



"""
Decorators
"""

# Standard Library
import time
from functools import wraps

# Django
from django.core.cache import cache

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.esi import fetch_esi_status
from app_utils.logging import LoggerAddTag

# AA TaxSystem
from taxsystem import __title__
from taxsystem.app_settings import IS_TESTING

logger = LoggerAddTag(get_extension_logger(__name__), __title__)

ESI_STATUS_ROUTE_RATE_LIMIT = 3
ESI_STATUS_KEY = "esi-is-available-status"


def get_esi_available_cache() -> bool:
    """Return True if ESI availability cache is present."""
    if cache.get(ESI_STATUS_KEY):
        return True
    return False


def when_esi_is_available(func):
    """
    Make sure the decorated task only runs when esi is available and store the result.
    Complete the task without running it when downtime is detected.
    Automatically disabled during tests.
    """

    @wraps(func)
    def outer(*args, **kwargs):

        # During tests we skip ESI checks
        if IS_TESTING or get_esi_available_cache():
            logger.debug("Skipping ESI check (testing mode or cache present).")
            return func(*args, **kwargs)

        # Check ESI status
        if fetch_esi_status().is_ok:
            logger.debug("ESI is available, proceeding.")
            cache.set(ESI_STATUS_KEY, "1", timeout=ESI_STATUS_ROUTE_RATE_LIMIT)
            return func(*args, **kwargs)
        return None  # function will not run

    return outer


def log_timing(logs):
    """
    A Decorator to log the time a function takes to run.
    """

    def decorator(func):
        def wrapper(*args, **kwargs):
            start_time = time.time()
            result = func(*args, **kwargs)
            end_time = time.time()
            logs.debug(
                "TIME: %s run for %s seconds with args: %s",
                end_time - start_time,
                func.__name__,
                args,
            )
            return result

        return wrapper

    return decorator



"""Shared ESI client for Tax System."""

# Alliance Auth
from esi.clients import EsiClientProvider

# AA TaxSystem
from taxsystem import __app_name_useragent__, __github_url__, __title__, __version__

esi = EsiClientProvider(
    ua_appname=__app_name_useragent__, ua_version=__version__, ua_url=__github_url__
)



# Django
from django.template.loader import render_to_string
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _

# AA TaxSystem
from taxsystem.models.tax import OwnerAudit, Payments


def get_manage_corporation(request, corporation_id) -> tuple[OwnerAudit | None, bool]:
    """Get Corporation and Permission"""
    perms = True
    try:
        corp = OwnerAudit.objects.get(corporation__corporation_id=corporation_id)
    except OwnerAudit.DoesNotExist:
        return None, None

    visible = OwnerAudit.objects.visible_to(request.user)
    if corp not in visible:
        perms = False
    return corp, perms


def get_corporation(request, corporation_id) -> OwnerAudit | None:
    """Get Corporation"""
    try:
        corp = OwnerAudit.objects.get(corporation__corporation_id=corporation_id)
    except OwnerAudit.DoesNotExist:
        return None

    # Check access
    visible = OwnerAudit.objects.visible_to(request.user)
    if corp not in visible:
        corp = None
    return corp


def get_manage_permission(request, corporation_id) -> bool:
    """Get Permission for Corporation"""
    perms = True

    try:
        corp = OwnerAudit.objects.get(corporation__corporation_id=corporation_id)
    except OwnerAudit.DoesNotExist:
        return False

    # Check access
    visible = OwnerAudit.objects.manage_to(request.user)
    if corp not in visible:
        perms = False
    return perms


def get_character_permissions(request, character_id) -> bool:
    """Get Permission for Character"""
    perms = True

    char_ids = request.user.character_ownerships.all().values_list(
        "character__character_id", flat=True
    )
    if character_id not in char_ids:
        perms = False
    return perms


def get_info_button(corporation_id, payment: Payments, request) -> mark_safe:
    details = generate_settings(
        title=_("Payment Details"),
        icon="fas fa-info",
        color="primary",
        text=_("View Payment Details"),
        modal="modalViewDetailsContainer",
        action=f"/taxsystem/api/corporation/{corporation_id}/character/{payment.character_id}/payment/{payment.pk}/view/details/",
        ajax="ajax_details",
    )
    button = generate_button(
        corporation_id=corporation_id,
        template="taxsystem/partials/form/button.html",
        queryset=payment,
        settings=details,
        request=request,
    )
    return button


def generate_button(
    corporation_id: int, template, queryset, settings, request
) -> mark_safe:
    """Generate a html button for the tax system"""
    return format_html(
        render_to_string(
            template,
            {
                "corporation_id": corporation_id,
                "queryset": queryset,
                "settings": settings,
            },
            request=request,
        )
    )


# pylint: disable=too-many-positional-arguments
def generate_settings(
    title: str, icon: str, color: str, text: str, modal: str, action: str, ajax: str
) -> dict:
    """Generate a settings dict for the tax system"""
    return {
        "title": title,
        "icon": icon,
        "color": color,
        "text": text,
        "modal": modal,
        "action": action,
        "ajax": ajax,
    }



# Alliance Auth
from allianceauth.eveonline.models import EveCharacter

# Alliance Auth (External Libs)
from app_utils.testing import (
    create_user_from_evecharacter,
)

# AA TaxSystem
# AA Tax System
from taxsystem.models.tax import OwnerAudit
from taxsystem.models.wallet import (
    CorporationWalletDivision,
    CorporationWalletJournalEntry,
)


def create_division(corporation: OwnerAudit, **kwargs) -> CorporationWalletDivision:
    """Create a CorporationWalletDivision"""
    params = {
        "corporation": corporation,
    }
    params.update(kwargs)
    division = CorporationWalletDivision(**params)
    division.save()
    return division


def create_wallet_journal_entry(**kwargs) -> CorporationWalletJournalEntry:
    """Create a CorporationWalletJournalEntry"""
    params = {}
    params.update(kwargs)
    journal_entry = CorporationWalletJournalEntry(**params)
    journal_entry.save()
    return journal_entry



"""TestView class."""

# Standard Library
import json
from http import HTTPStatus
from unittest.mock import Mock, patch

# Django
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase
from django.urls import reverse

# Alliance Auth (External Libs)
from app_utils.testing import create_user_from_evecharacter

# AA TaxSystem
from taxsystem import views
from taxsystem.models.tax import Payments, PaymentSystem
from taxsystem.tests.testdata.generate_owneraudit import (
    create_owneraudit_from_user,
)
from taxsystem.tests.testdata.generate_payments import (
    create_payment,
    create_payment_system,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "taxsystem.views"


class TestAccountView(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            character_id=1001,
            permissions=[
                "taxsystem.basic_access",
                "taxsystem.manage_own_corp",
            ],
        )
        cls.audit = create_owneraudit_from_user(cls.user)
        cls.no_audit_user, _ = create_user_from_evecharacter(
            character_id=1002,
            permissions=[
                "taxsystem.basic_access",
                "taxsystem.manage_own_corp",
            ],
        )
        cls.no_permission_user, _ = create_user_from_evecharacter(
            character_id=1003,
            permissions=[
                "taxsystem.basic_access",
            ],
        )
        cls.payment_system = create_payment_system(
            name=cls.user.username,
            owner=cls.audit,
            user=cls.user,
        )
        cls.payment_system_inactive = create_payment_system(
            name=cls.no_permission_user.username,
            owner=cls.audit,
            user=cls.no_permission_user,
            status=PaymentSystem.Status.DEACTIVATED,
        )

    @patch(MODULE_PATH + ".messages")
    def test_should_be_ok(self, mock_messages):
        """Test account view."""
        # given
        request = self.factory.get(
            reverse(
                "taxsystem:account",
            )
        )

        request.user = self.user

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)

        # when
        response = views.account(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        mock_messages.error.assert_not_called()



# Standard Library
from io import StringIO
from unittest.mock import patch

# Django
from django.core.management import call_command
from django.utils import timezone

# Alliance Auth (External Libs)
from app_utils.testing import NoSocketsTestCase, create_user_from_evecharacter
from eveuniverse.models import EveEntity

# AA TaxSystem
from taxsystem.models.tax import Payments, PaymentSystem

# AA Tax System
from taxsystem.tests.testdata.generate_owneraudit import create_owneraudit_from_user
from taxsystem.tests.testdata.generate_payments import (
    create_payment,
    create_payment_system,
)
from taxsystem.tests.testdata.generate_walletjournal import (
    create_division,
    create_wallet_journal_entry,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse

COMMAND_PATH = "taxsystem.management.commands.taxsystem_migrate_payments"


class TestMigratePayments(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
        )
        cls.audit = create_owneraudit_from_user(cls.user)

        cls.eve_character_first_party = EveEntity.objects.get(id=2001)
        cls.eve_character_second_party = EveEntity.objects.get(id=1001)

        cls.division = create_division(
            corporation=cls.audit,
            division_id=1,
            name="Main Division",
            balance=1000000,
        )

        cls.journal_entry = create_wallet_journal_entry(
            division=cls.division,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
            reason="Test Journal Entry",
            ref_type="tax_payment",
            first_party=cls.eve_character_first_party,
            second_party=cls.eve_character_second_party,
        )

        cls.payment_system = create_payment_system(
            name=cls.character_ownership.character.character_name,
            owner=cls.audit,
            user=cls.user,
            status=PaymentSystem.Status.ACTIVE,
            deposit=0,
            last_paid=(timezone.now() - timezone.timedelta(days=30)),
        )

        cls.payments = create_payment(
            name=cls.character_ownership.character.character_name,
            account=cls.payment_system,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
            reason="Tax Payment",
            request_status=Payments.RequestStatus.PENDING,
            reviser="",
        )

    def test_should_migrate(self):
        # when
        out = StringIO()
        call_command("taxsystem_migrate_payments", stdout=out)
        output = out.getvalue()

        # then
        self.assertIn(
            "Migration report for Hell RiderZ - Status: incomplete: 1 entries migrated.",
            output,
        )



"""General Model"""

# Standard Library
from dataclasses import dataclass
from typing import Any, NamedTuple

# Django
from django.db import models
from django.utils.translation import gettext_lazy as _


class General(models.Model):
    """General model for app permissions"""

    class Meta:
        managed = False
        permissions = (
            ("basic_access", _("Can access the Tax System")),
            ("create_access", _("Can add Corporation")),
            ("manage_own_corp", _("Can manage own Corporation")),
            ("manage_corps", _("Can manage all Corporations")),
        )
        default_permissions = ()


class UpdateSectionResult(NamedTuple):
    """A result of an attempted section update."""

    is_changed: bool | None
    is_updated: bool
    data: Any = None


@dataclass(frozen=True)
class _NeedsUpdate:
    """An Object to track if an update is needed."""

    section_map: dict[str, bool]

    def __bool__(self) -> bool:
        """Check if any section needs an update."""
        return any(self.section_map.values())

    def for_section(self, section: str) -> bool:
        """Check if an update is needed for a specific section."""
        return self.section_map.get(section, False)



# Standard Library
from unittest.mock import patch

# Django
from django.test import override_settings
from django.utils import timezone

# Alliance Auth (External Libs)
from app_utils.testing import NoSocketsTestCase, create_user_from_evecharacter
from eveuniverse.models import EveEntity

# AA TaxSystem
from taxsystem.models.tax import Members
from taxsystem.tests.testdata.esi_stub import esi_client_stub
from taxsystem.tests.testdata.generate_owneraudit import create_owneraudit_from_user
from taxsystem.tests.testdata.generate_payments import (
    create_member,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "taxsystem.managers.tax_manager"


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
@patch(MODULE_PATH + ".esi")
@patch(MODULE_PATH + ".etag_results")
@patch(MODULE_PATH + ".EveEntity.objects.bulk_resolve_names")
@patch(MODULE_PATH + ".logger")
class TestMembersManager(NoSocketsTestCase):
    """Test Members Manager for Corporation Members."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
        )
        cls.audit = create_owneraudit_from_user(cls.user)

        cls.member = create_member(
            owner=cls.audit,
            character_id=1001,
            character_name="Member 1",
            status=Members.States.ACTIVE,
        )

        cls.member_2 = create_member(
            owner=cls.audit,
            character_id=1004,
            character_name="Member 4",
            status=Members.States.ACTIVE,
        )

    def test_update_members(self, mock_logger, mock_bulk_resolve, mock_etag, mock_esi):
        # given
        mock_esi.client = esi_client_stub
        mock_etag.side_effect = lambda ob, token, force_refresh=False: ob.results()

        mock_bulk_resolve.return_value.to_name.side_effect = (
            "Member 1",
            "Member 2",
            "Member 3",
        )

        self.audit.update_members(force_refresh=False)

        obj = self.audit.ts_members.get(character_id=1001)
        self.assertEqual(obj.character_name, "Member 1")

        obj = self.audit.ts_members.get(character_id=1002)
        self.assertEqual(obj.character_name, "Member 2")

        obj = self.audit.ts_members.get(character_id=1003)
        self.assertEqual(obj.character_name, "Member 3")

        mock_logger.info.assert_called_with(
            "Corp %s - Old Members: %s, New Members: %s, Missing: %s",
            self.audit.corporation.corporation_name,
            1,
            2,
            1,
        )



# Generated by Django 4.2.11 on 2025-04-01 22:04

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0003_alter_general_options_alter_paymenthistory_comment"),
    ]

    operations = [
        migrations.AddField(
            model_name="owneraudit",
            name="last_update_payday",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="paymentsystem",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("inactive", "Inactive"),
                    ("deactivated", "Deactivated"),
                    ("missing", "Missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
    ]



"""TestView class."""

# Standard Library
from http import HTTPStatus
from unittest.mock import Mock, patch

# Django
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase
from django.urls import reverse

# Alliance Auth (External Libs)
from app_utils.testing import create_user_from_evecharacter

# AA TaxSystem
from taxsystem import views

# AA Taxsystem
from taxsystem.tests.testdata.generate_owneraudit import (
    create_owneraudit_from_user,
    create_user_from_evecharacter_with_access,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse

INDEX_PATH = "taxsystem.views"


@patch(INDEX_PATH + ".messages")
class TestViewAdministrationAccess(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1002
        )
        cls.superuser, cls.character_ownership = (
            create_user_from_evecharacter_with_access(1001)
        )

    def test_admin(self, mock_messages):
        """Test admin access."""
        # given
        self.superuser.is_superuser = True
        self.superuser.save()

        request = self.factory.get(reverse("taxsystem:admin"))
        request.user = self.superuser
        # when
        response = views.admin(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Administration")

    def test_admin_no_access(self, mock_messages):
        """Test admin access."""
        # given
        request = self.factory.get(reverse("taxsystem:admin"))
        request.user = self.user

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # when
        response = views.admin(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertTrue(mock_messages.error.called)

    def test_admin_clear_all_etags(self, mock_messages):
        """Test clear all etags."""
        # given
        self.superuser.is_superuser = True
        self.superuser.save()
        request = self.factory.post(
            reverse("taxsystem:admin"), data={"run_clear_etag": True}
        )
        request.user = self.superuser

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # when
        response = views.admin(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        mock_messages.info.assert_called_once_with(request, "Queued Clear All ETags")

    def test_force_refresh(self, mock_messages):
        """Test force refresh."""
        # given
        self.superuser.is_superuser = True
        self.superuser.save()
        request = self.factory.post(
            reverse("taxsystem:admin"), data={"force_refresh": True}
        )
        request.user = self.superuser

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # when
        response = views.admin(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_run_taxsystem_updates(self, mock_messages):
        """Test run char updates."""
        # given
        self.superuser.is_superuser = True
        self.superuser.save()
        request = self.factory.post(
            reverse("taxsystem:admin"), data={"run_taxsystem_updates": True}
        )
        request.user = self.superuser

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        # when
        response = views.admin(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        mock_messages.info.assert_called_once_with(
            request, "Queued Update All Taxsystem"
        )


class TestViewAccess(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )

        cls.superuser, cls.character_ownership = (
            create_user_from_evecharacter_with_access(1002)
        )

        cls.manage_user = create_user_from_evecharacter(
            1003,
            permissions=[
                "taxsystem.basic_access",
                "taxsystem.manage_own_corp",
            ],
        )[0]

        cls.audit = create_owneraudit_from_user(cls.user)

    def test_view_index(self):
        """Test view taxsystem index."""
        # given
        request = self.factory.get(reverse("taxsystem:index"))
        request.user = self.user
        # when
        response = views.index(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)

    def test_view_administration(self):
        """Test view administration."""
        # given
        request = self.factory.get(
            reverse(
                "taxsystem:administration",
                args=[2003],
            )
        )
        request.user = self.manage_user
        # when
        response = views.administration(request, 2003)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Administration")

    def test_view_payments(self):
        """Test view payments."""
        # given
        request = self.factory.get(
            reverse(
                "taxsystem:payments",
                args=[2001],
            )
        )
        request.user = self.user
        # when
        response = views.payments(request, 2001)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Payments")

    def test_view_own_payments(self):
        """Test view own payments."""
        # given
        request = self.factory.get(
            reverse(
                "taxsystem:own_payments",
                args=[2001],
            )
        )
        request.user = self.user
        # when
        response = views.own_payments(request, 2001)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Own Payments")

    def test_view_faq(self):
        """Test view FAQ."""
        # given
        request = self.factory.get(
            reverse(
                "taxsystem:faq",
                args=[2001],
            )
        )
        request.user = self.user
        # when
        response = views.faq(request, 2001)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "FAQ")

    @patch(INDEX_PATH + ".messages")
    def test_view_account(self, mock_messages):
        """Test view account."""
        # given
        request = self.factory.get(
            reverse(
                "taxsystem:account",
            )
        )
        request.user = self.user

        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)

        # when
        response = views.account(request)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        mock_messages.error.assert_called()

    def test_view_manage_filters(self):
        """Test view manage filters."""
        # given
        self.superuser.is_superuser = True
        self.superuser.save()
        request = self.factory.get(
            reverse(
                "taxsystem:manage_filter",
                args=[2001],
            )
        )
        request.user = self.superuser
        # when
        response = views.manage_filter(request, corporation_id=2001)
        # then
        self.assertEqual(response.status_code, HTTPStatus.OK)
        self.assertContains(response, "Manage Filters")



"""Models for Filters."""

# Django

# Django
from django.db import models
from django.db.models import QuerySet
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

# AA TaxSystem
from taxsystem.models.tax import OwnerAudit, Payments
from taxsystem.models.wallet import CorporationWalletJournalEntry


class JournalFilterSet(models.Model):
    owner = models.ForeignKey(
        OwnerAudit, on_delete=models.CASCADE, related_name="ts_filter_set"
    )
    name = models.CharField(max_length=100, unique=True)
    description = models.CharField(max_length=255, blank=True)
    enabled = models.BooleanField(default=True)

    def __str__(self):
        return self.name

    @property
    def is_active(self) -> bool:
        return self.enabled

    @property
    def is_active_html(self) -> mark_safe:
        if self.enabled:
            return mark_safe('<i class="fa-solid fa-check"></i>')
        return mark_safe('<i class="fa-solid fa-times"></i>')

    def filter(self, payments: Payments) -> models.QuerySet[Payments]:
        if self.is_active:
            for f in self.ts_filters.all():
                payments = f.apply_filter(payments)
            return payments
        return Payments.objects.none()

    def filter_contains(
        self, payments: Payments
    ) -> models.QuerySet[Payments]:  # not implemented yet
        if self.is_active:
            for f in self.ts_filters.all():
                payments = f.apply_contains(payments)
            return payments
        return Payments.objects.none()

    class Meta:
        verbose_name = _("Journal Filter Set")
        verbose_name_plural = _("Journal Filter Sets")
        default_permissions = ()


class JournalFilter(models.Model):
    class FilterType(models.TextChoices):
        REASON = "reason", _("Reason")
        AMOUNT = "amount", _("Amount")

    filter_set = models.ForeignKey(
        JournalFilterSet, on_delete=models.CASCADE, related_name="ts_filters"
    )
    filter_type = models.CharField(max_length=20, choices=FilterType.choices)
    value = models.CharField(max_length=255)

    def __str__(self):
        return f"{self.filter_type}: {self.value}"

    def apply_filter(
        self, qs: QuerySet[CorporationWalletJournalEntry]
    ) -> QuerySet[CorporationWalletJournalEntry]:
        if self.filter_type == JournalFilter.FilterType.REASON:
            return qs.filter(reason=self.value)
        if self.filter_type == JournalFilter.FilterType.AMOUNT:
            return qs.filter(amount=self.value)
        # weitere Felder
        return qs

    def apply_contains(
        self, qs: QuerySet[CorporationWalletJournalEntry]
    ) -> QuerySet[CorporationWalletJournalEntry]:
        if self.filter_type == JournalFilter.FilterType.REASON:
            return qs.filter(reason__icontains=self.value)
        if self.filter_type == JournalFilter.FilterType.AMOUNT:
            return qs.filter(amount__gte=self.value)
        # weitere Felder
        return qs

    class Meta:
        constraints = [
            models.UniqueConstraint(
                fields=["filter_set", "value"], name="unique_filter_value_per_set"
            )
        ]
        default_permissions = ()
